C*TODFFT.FOR********************************************************
C
C	TWO-DIMENSIONAL FOURIER TRANSFORM SUBROUTINE FOR IMAGE
C	PROCESSING. DOES BOTH FOWARD & INVERSE TRANSFORMS
C	USES LYNN TENEYCK'S MIXED-RADIX ROUTINES
C	THUS THE ONLY RESTRICTION IS THAT THE IMAGE SIZE BE
C	AN EVEN NUMBER AND HAVE NO PRIME FACTORS LARGER THAN 19!!
C
C	IDIR =  0	FOWARD  TRANSFORM  :  exp(+2PIirs)
C	IDIR =  1	INVERSE TRANSFORM  :  exp(-2PIirs)
C	IDIR = -1	INVERSE TRANSFORM BUT NO COMPLEX CONJUGATE
C
C	DATA SET UP AS NY ROWS OF NX NUMBERS
C	NOTE NX,NY ALWAYS REFER TO THE REAL-SPACE IMAGE SIZE
C
C	NX,NY IMAGE IS TRANSFORMED IN-PLACE INTO NY STRIPS OF
C	NX/2 + 1 COMPLEX FOURIER COEFFICIENTS
C	THE ORIGIN IS LOCATED AT THE FIRST POINT!!!
C
C	ARRAY MUST BE DIMENSIONED TO ALLOW FOR EXTRA STRIP OF COMPLEX
C	NUMBERS ON OUTPUT.
C	THUS FOR A 300X400 TRANSFORM, ARRAY MUST BE DIMENSIONED:
C	REAL ARRAY(302,400)
C
C	A NORMALIZATION FACTOR IS APPLIED DURING BOTH  TRANSFORMATIONS
C
C	VERSION 1.00	OCT 11 1981		DAA
C	VERSION 1.02	APR 19 1982		DAA
C	VERSION 1.03	NOV 23 1982		DAA
C
        SUBROUTINE TODFFT(ARRAY,NX,NY,IDIR)
        DIMENSION ARRAY(*),IDIM(5)
C
        NXO2 = NX/2
        IF (2*NXO2 .EQ. NX) GOTO 1
        WRITE(6,1000) NX
1000    FORMAT(' TODFFT: NX= ',I7,' MUST BE EVEN!!!')
        STOP
1       NXP2 = NX + 2
        NXT = NXP2*NY
        NXT1 = NXT - 1
        ONEVOL = SQRT(1.0/(NX*NY))
        IF (IDIR .NE. 0) GOTO 50
C
C********      FOWARD TRANSFORMS COME HERE       ******************
C
C
C  SET UP FOR FIRST DIMENSION OF TRANSFORM
C
        IDIM(1) = NXP2*NY
        IDIM(2) = 2
        IDIM(3) = IDIM(1)
        IDIM(4) = IDIM(1)
        IDIM(5) = NXP2
C
        CALL REALFT(ARRAY(1),ARRAY(2),NXO2,IDIM)
C
C  SET UP FOR SECOND DIMENSION OF TRANSFORM
C
        IDIM(2) = NXP2
        IDIM(4) = IDIM(2)
        IDIM(5) = 2
C
        CALL CMPLFT(ARRAY(1),ARRAY(2),NY,IDIM)
C
C   TAKE COMPLEX CONJUGATE (TO MAKE PROPER FOWARD TRANSFORM)& SCALE BY 1/VOLUME
C
        DO 100 J = 1,NXT1,2
          ARRAY(J) = ARRAY(J)*ONEVOL
          ARRAY(J+1) = -ARRAY(J+1)*ONEVOL
100     CONTINUE
C
        RETURN
C
C**********        INVERSE TRANSFORM     *******************
C
C
C  SET UP FOR FIRST DIMENSION OF TRANSFORM
C
50      NXM1 = NX - 1
        IDIM(1) = NXP2*NY
        IDIM(2) = NXP2
        IDIM(3) = IDIM(1)
        IDIM(4) = IDIM(2)
        IDIM(5) = 2
C
C   TAKE COMPLEX CONJUGATE TO DO INVERSE & SCALE BY 1/VOLUME
C
        IF (IDIR .EQ. 1) GOTO 60
        DO 300 J = 1,NXT1,2
          ARRAY(J) = ARRAY(J)*ONEVOL
          ARRAY(J+1) = -ARRAY(J+1)*ONEVOL
300     CONTINUE
        GOTO 70
C
C   IDIR = 1 JUST SCALE BY 1/VOLUME (FOR STANDARD INVERSE TRANSFORM)
C
60      DO 400 J = 1,NXT1,2
          ARRAY(J) = ARRAY(J)*ONEVOL
          ARRAY(J+1) = ARRAY(J+1)*ONEVOL
400     CONTINUE
C
70      CALL CMPLFT(ARRAY(1),ARRAY(2),NY,IDIM)
C
C  SET UP FOR SECOND DIMENSION OF TRANSFORM
C
        IDIM(2) = 2
        IDIM(4) = IDIM(1)
        IDIM(5) = NXP2
C
C    CHANGE DATA STORAGE MODE COMPLEX CONJUGATE DONE BY HERMFT
C
        INDEX = 2
        DO 500 IY = 1,NY
          ARRAY(INDEX) = ARRAY(NXM1 + INDEX)
          INDEX = INDEX + NXP2
500     CONTINUE
C
        CALL HERMFT(ARRAY(1),ARRAY(2),NXO2,IDIM)
C
        RETURN
        END
