.TH imodenv 1 3.10 BL3DEMC
.na
.nh

.SH NAME
Description of IMOD environment variables

.SH DESCRIPTION
This page describes all of the environment variables that can be set for use
with IMOD programs.

.SS User-level variables

.TP
.B AUTODOC_DIR
Alternative directory to search for an autodoc file when a program uses PIP
for input.
.TP
.B ETOMO_LOG_DIR
Directory into which eTomo will place error log files named by date and time,
instead of into $HOME/.etomologs.  Set this to a non-existent name (e.g.,
stuffAndNonsense) to have sequentially numbered error logs kept in the
directory from which eTomo was started.
.TP
.B ETOMO_LOGS_TO_RETAIN
Set this to the number of log files named by date and time to retain in the 
directory where they are kept.  The default is 31 if this variable is not set.
.TP
.B ETOMO_MEM_LIM
Memory limit for running etomo.  Define this as a number of megabytes followed
by the letter "m".  The default of 128m is set in the etomo startup script.
Larger values will allow larger final alignment logs to be displayed.
.TP
.B IMOD_AUTOSAVE
Interval at which 3dmod should autosave the model, in minutes (superceded by
setting in 3dmod preferences).
.TP
.B IMOD_AUTOSAVE_DIR
Directory into which 3dmod should autosave the model (superceded by
setting in 3dmod preferences).
.TP
.B IMOD_BRIEF_HEADER
If this variable is set, all Fortran programs except header(1) will output
a brief header with the dimensions, pixel size, mode, minimum, maximum,
and mean value, and the first and last title.
.TP
.B IMOD_CALIB_DIR
Directory in which IMOD-related information can live, unaffected by upgrading
the main IMOD directory.  Set by startup scripts to /usr/local/ImodCalib.
.TP
.B IMOD_CSHELL
Variable specifying a command to use when running csh/tcsh on a script.
Used by Bead Fixer and could be used elsewhere.  This variable can contain a
path to the C shell executable plus flags.  The -f flag is always added.
.TP
.B IMOD_DIR
The path to the top IMOD directory; set by startup scripts, or can be set
before startup script is run.
.TP
.B IMOD_DUMP_FSCACHE
Makes 3dmod tell the system to dismiss the file system cache while
loading a file on Linux.  This behavior was originally the default because,
on early 64-bit systems with files bigger
than about half of the system memory capacity,
the system would swap out data within 3dmod while loading.
This bad system behavior is no longer observed in testing on FC4, RHEL4 and 
RHEL5.  Set this variable (no value needed) for working on systems that
show this problem.
.TP
.B IMOD_JAVADIR
Location of Java to be used to run eTomo; $IMOD_JAVADIR/bin is put on the
front of the path in the etomo startup script.
.TP
.B IMOD_NO_IMAGE_BACKUP
If this variable is set (no value needed), virtually all programs that create
image files will no longer back up an existing file by adding ~ to the name.
Instead, they will overwrite the file.
.TP
.B IMOD_PLUGIN_DIR
Main directory for IMOD plugins; set by startup scripts to be inside IMOD.
.TP
.B IMOD_PROCESSORS
Number of processors in the computer; used by slicer to decide how many threads
to run in parallel.  Also used to create a parallel processing table when
cpu.adoc is not found.
.TP
.B IMOD_PS_FONT
Name of a font (instead of Helvetica) to use in the Postscript output from 
various graphics programs; Arial is good for going to Illustrator.
.TP
.B IMOD_QTLIBDIR
Directory in which IMOD's copy of Qt lives, if any.  Set by IMOD startup
scripts.
.TP
.B IMOD_STEREO_COMMAND
When using hardware stereo, system command for switching the monitor to
display left and right buffers in overlapping fashion.
.TP
.B IMOD_STEREO_RESTORE
System command for switching the monitor back to normal mode.
.TP
.B IMOD_STEREO_TBOFFSET
Default value of vertical offset when using top/bottom stereo display.
.TP
.B IMOD_TMPDIR
Path for temporary directory used by scripts.  On Windows, this should be in
Cygwin format, not in Windows format.
.TP
.B IMOD_USE_GPU
This variable will tell the Tilt program (and eventually perhaps other
programs) to use the GPU on a CUDA-capable NVIDIA graphics card.  Set the
variable to 0 to use the best available GPU, or to a positive number to
specify the GPU to use.  If there is only one GPU, 0 and 1 have the same
effect.  This setting is overridden by the UseGPU entry to Tilt, if any.
.TP
.B PIP_PRINT_ENTRIES
Set this variable to 1 to enable printing of all options entered into a program
using PIP input or 0 to disable it.  The variable is automatically
set to 1 when running programs through command files (i.e., vmstocsh and
vmstopy both add lines to their output to set the variable to 1).
Thus, if you leave the variable unset in your environment, entries will be
printed in programs run through "subm" but not when programs are run at the
command line.  If you want the output when running at the command line, set
the variable to 1 in your environment.  To avoid this output when running 
a command file, add a line
.br
    $setenv PIP_PRINT_ENTRIES 0
.br
at the beginning of the file.
.TP
.B PLAX_REDRAWS
If there is trouble with the "plax" window used by Fortran graphics programs
not updating properly, set this variable to the number of times for it be
drawn (try 2 or 3).  These programs include nda, sda, mtk, genhstplt,
mtoverlap, mtpairing, avgstatplot, and filterplot.
.TP
.B RAPTOR_BIN
Directory where alternative RAPTOR executables are located; leave this
undefined to run the executables in IMOD.
.TP
.B READ_MODE0_SIGNED
Set this variable non-zero to override the default for whether an MRC file in
mode 0 should be read as signed or unsigned bytes.  Potential values are:
.br
   -2   to read all mode 0 files as unsigned, even ones with an IMOD stamp
.br
   -1   to read mode 0 files lacking an IMOD stamp as unsigned
.br
   1    to read mode 0 files lacking an IMOD stamp as signed
.br
   -2   to read all mode 0 files as signed, even ones with an IMOD stamp
.br
The values of -1 and 1 could be useful if a file header has incorrect entries
for the minimum and maximum.  Values of -2 or 2 would be needed if other
software changed the representation of bytes in a file without modifying the
IMOD stamp and flag values. 
.TP
.B RUNCMD_VERBOSE
Set this variable to 1 to have the runcmd function called by Python programs
print each command, input to the command, and the output of the command unless
the output was already printed on standard out.
.TP
.B SUBM_LOG_TYPE
Set this variable to a positive value up to 4 to have subm (submfg) produce
sequentially numbered log files with the given number of digits, or to -1 to
-3 to have a date-time stamp attached the log file name.
.TP
.B SUBM_MESSAGE
Set this variable to an alternative message to be used when subm (submfg)
completes execution of a command file.
.TP
.B WRITE_MODE0_SIGNED
Set this variable to 1 to have all MRC files in mode 0 written as signed
bytes.  In some future IMOD version, this may be the default behavior, and
then the variable can be set to 0 to have files written as unsigned bytes.

.SS Build and Test Variables

.TP
.B CUDA_DIR
Directory with CUDA toolkit, must be defined to enable building with CUDA.
.TP
.B CUDA_BIN_PATH
On Windows, this must be defined as the location of CUDA binaries if they are
not in $CUDA_DIR/bin
.TP
.B CUDA_LIB_PATH
On Windows, this must be defined as the location of CUDA libraries if they are
not in $CUDA_DIR/lib
.TP 
.B IMOD_TEST_SECTION
This is passed to the uitest script when running eTomo tests.  It is used to
choose one of the tests in uitest.adoc.
.TP
.B IMOD_UITEST_DATA
When running eTomo tests, the root directory for data files.
.TP
.B IMOD_UITEST_SCRIPT 
This is used in uitest.make.  It should be set to the
location of the uitest script
.TP
.B IMOD_UITEST_SOURCE
When running eTomo tests, the location of the uitest.adoc and all
autodocs it refers to.
.TP
.B JAVA_DIR
Location of Java SDK to be used by the IMOD build and test script.
.TP
.B MSVCREDIST
Location of Microsoft Visual C redistributable run-time libraries that are
to be copied when building a distribution.  When this is defined, manifests
are also added to all executables.
.TP
.B QTDIR
Location of Qt, or at least of Qt bin directory if include is elsewhere.
.TP
.B SOLVEMATCH_TEST
Set this variable to prevent Solvematch from exiting with error when given a
data set with relative coordinates.

