.na
.nh
.TH imodjoin 1 2.50 BL3DEMC
.SH NAME
imodjoin \- Combine two or more IMOD models into one.
.SH SYNOPSIS
imodjoin [options] model_1 [-o list] model_2 [[-o list] model_3 ...] out_model
.SH DESCRIPTION
Combines the contents of two or more models, or a subset of the objects from
them, into
a single model.  With no options, the program will produce a model with all
of the objects from the first model, followed by all of the objects from the
second model, then all of the objects from any additional models.
.P
The program will transform models if necessary to bring them to a common
coordinate system, using image reference information stored in the model.
The most common case requiring this is when the models
have last been loaded on different subsets of the same image file.  The
default assumption is that the models were loaded on either the same image
file or image files with congruent coordinate systems.  For example, an image
file produced by Trimvol(1) or Rotatevol(1) has its header entries adjusted so
that its coordinate system is congruent with that of the original image file.
Squeezevol(1) also produces a file with a congruent coordinate system.
When joining models coming from such files, the program will take all
coordinate differences between the models into account: shifts, rotations, and
scaling.  This is not always appropriate, so the 
.B -d
option is available to indicate that models came from different volumes.  Some
examples of different or non-congruent volumes are
tomograms from serial sections, single and dual-axis tomograms of the same
specimen, and two files of the same volume in which Y and Z are flipped.  In
such cases, the program will simply shift each model, if necessary, to undo
the effects of being loaded onto a subset of its respective image file; it
will also scale all models to a common scale unless the 
.B -s
option is given.  The resulting model will be given image reference
information so that it will display correctly on the volume
that the first model came from, unless a specific image file is identified
with the
.B -i
option, in which case the reference information will match that image.
.P
The program will give a warning when it does not find adequate image reference
information in the models.  Despite these warnings, the joining should work
as long as the models are based on the same image stack, or on image stacks
with similar coordinate systems (e.g., identical pixel spacings and origins).
The warnings could be avoided by loading the model together with a relevant
image file into 3dmod and resaving the model.

.SS Options
All options except for
.B -o
must be entered before the first input model; 
.B -o
may be entered before each input model.
.TP
.B -o <list of objects>
This option can be used before a model file to specify a list of objects that 
should be taken from that file.
Each list of objects 
can consist of comma-separated ranges of numbers, for example
1-3,6-9,13,15.
.TP
.B -r  <list of objects>
This option can be used to replace the listed objects in the first model 
with objects from the second model.  In this case, the 
.B -o
option cannot be used for the first model; the output model will have the same
number and ordering of objects as the first model, with the specified ones 
replaced.  This option can be used only with two input models.
If no 
.B -o
option is used for the second model, the objects in the list for the first 
model will be replaced by objects 1, 2, 3, etc. from the second model.  If
the
.B -o 
option is used for the second model, then each of the objects in the 
replacement list for the first model will be replaced by the corresponding
object in the list for the second model.
.TP
.B -c
Change colors of the objects being added to the first model.  If objects are
being added without replacement, then they will be changed to the colors
automatically assigned to new objects.  If objects are replacing ones in the
first model, then they will be given the same color as the respective objects
that they are replacing.  Without this option, objects will retain their
original colors, which is especially annoying when there are just one or two
objects from each model.
.TP
.B -d
Models are from different volumes rather than from the same volume or volumes
with congruent coordinate systems (see above).
.TP 
.B -i <filename>
With this option, the header information in the given image file will be
taken as the reference for all models, and the resulting model will display
correctly on this image file.  Otherwise, the first model is taken as the
reference.  This option implies 
.B -d.
.TP
.B -s
Do not adjust for scale (pixel spacing) differences between models from
different volumes.  This option might be appropriate either if the pixel
size differences in the image file headers do not accurately reflect 
different scalings between the images, or if the scale differences are being
handled separately, such as by an alignment transformation.
.TP
.B -f
Retain (or restore) the original Y-Z flipped state of each model.  Without this
option, the program will put all models into the same flipped state before
joining.  This option would be appropriate if one model was built on a
tomogram that was not flipped (and thus is loaded into 3dmod with flipping)
and another model was built on a flipped tomogram (no flipping needed in
3dmod).
.TP
.B -n
Do not transform models at all.  This option will prevent all of the behavior
described above.
.SH AUTHORS
.nf
Jim Kremer and David Mastronarde
.fi
.SH SEE ALSO
3dmod(1), imodextract(1)
.SH BUGS
Email bug reports to mast@colorado.edu.
